/*
 * Copyright 2011 Daniel Josefsson
 *
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package se.dannej.fakehttpserver;

import static se.dannej.fakehttpserver.Header.Builder.aHeader;

import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletRequestWrapper;

import org.apache.commons.io.IOUtils;

import se.dannej.fakehttpserver.expect.matcher.RequestMatcher;

/**
 * A {@link HttpServletRequest} that wraps the original {@link HttpServletRequest}, adding
 * some helper methods (e.g. {@link #getContent()} and {@link #getHeadersWithName()}).
 */
public class FakeHttpServletRequest extends HttpServletRequestWrapper {
	private Map<String, List<Header>> headers;
	private byte[] content;

	public FakeHttpServletRequest(HttpServletRequest request) {
		super(request);
	}
	
	/**
	 * Returns the content as a byte array. This is added to enable having more than one expectation
	 * checking the content (otherwise the first content-reading {@link RequestMatcher} would drain
	 * the request {@link InputStream}). N.B. use this instead of {@link #getInputStream()}!
	 * @return the request content
	 * @throws IOException 
	 */
	public byte[] getContent() throws IOException {
		if (content == null) {
			content = IOUtils.toByteArray(super.getInputStream());
		}
		return content;
	}
	
	/**
	 * Returns the request headers grouped by their name. Each entry in the map contains a list of
	 * the headers having the same name.
	 * @return the request headers
	 */
	public Map<String, List<Header>> getHeaders() {
		if (headers == null) {
			populateHeaders();
		}

		return Collections.unmodifiableMap(headers);
	}
	
	/**
	 * Returns a list of headers having the given name. If there are no headers with the given name,
	 * then an empty list is returned.
	 * @param name header name
	 * @return a list containing all headers having the given name
	 */
	public List<Header> getHeadersWithName(String name) {
		List<Header> headersWithMatchingName = getHeaders().get(name);
		
		if (headersWithMatchingName != null) {
			return headersWithMatchingName;
		} else {
			return Collections.<Header>emptyList();
		}
	}

	
	private void populateHeaders() {
		headers = new HashMap<String, List<Header>>();
		
		Enumeration<String> headerNames = getHeaderNames();
		while (headerNames.hasMoreElements()) {
			String headerName = headerNames.nextElement();
			List<Header> headersWithSameName = new ArrayList<Header>();
			
			Enumeration<String> headerValues = getHeaders(headerName);
			while (headerValues.hasMoreElements()) {
				headersWithSameName.add(aHeader()
						.withName(headerName)
						.withValue(headerValues.nextElement())
						.build());
			}
			
			headers.put(headerName, headersWithSameName);
		}
	}
}
