/*
 * Copyright 2011 Daniel Josefsson
 *
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package se.dannej.fakehttpserver.expect;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletResponse;

import org.hamcrest.Description;
import org.hamcrest.Matcher;

import se.dannej.fakehttpserver.FakeHttpServer;
import se.dannej.fakehttpserver.FakeHttpServletRequest;
import se.dannej.fakehttpserver.FakeHttpServletResponse;
import se.dannej.fakehttpserver.expect.cardinality.Cardinalities;
import se.dannej.fakehttpserver.expect.cardinality.Cardinality;

/**
 * An {@link Expectation} for a {@link FakeHttpServer}. Created by {@link Requests}.
 */
public class Expectation {
	private Cardinality cardinality;
	private List<Matcher<FakeHttpServletRequest>> requestMatchers = new ArrayList<Matcher<FakeHttpServletRequest>>();
	private List<ResponseAction> actions = new ArrayList<ResponseAction>();

	Expectation() {
	}

	List<Matcher<FakeHttpServletRequest>> getRequestMatchers() {
		return requestMatchers;
	}
	
	void addRequestMatcher(Matcher<FakeHttpServletRequest> requestMatcher) {
		this.requestMatchers.add(requestMatcher);
	}
	
	public boolean handle(FakeHttpServletRequest request, FakeHttpServletResponse response) {
		if (expectationsAreMet(request)) {
			cardinality.invoked();
			for (ResponseAction action : actions) {
				action.applyTo(request, response);
			}
			
			flushResponse(response);
			return true;
		}
		return false;
	}

	/**
	 * Set how expected a request is (e.g. once, never)
	 * @param cardinality
	 * @see Cardinalities
	 */
	void setcardinality(Cardinality cardinality) {
		this.cardinality = cardinality;
	}

	/**
	 * Add an action that should be taken if expectations are met.
	 * @param action action to take
	 */
	void addAction(ResponseAction action) {
		this.actions.add(action);
	}

	/**
	 * @param expectationDescription
	 * @return true if expectation has been met
	 */
	public boolean isSatisfied(Description expectationDescription) {
		expectationDescription.appendText("expected ");
		expectationDescription.appendDescriptionOf(cardinality);
		expectationDescription.appendValueList("", ",  ",  "", requestMatchers);
		
		return cardinality.isSatisfied();
	}

	
	private boolean expectationsAreMet(FakeHttpServletRequest request) {
		for (Matcher<FakeHttpServletRequest> requestMatcher : requestMatchers) {
			if (!requestMatcher.matches(request)) {
				return false;
			}
		}
		return true;
	}

	private void flushResponse(HttpServletResponse response) {
		try {
			response.flushBuffer();
		} catch (IOException e) { }
	}
}
