/*
 * Copyright 2011 Daniel Josefsson
 *
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package se.dannej.fakehttpserver.expect;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import javax.servlet.http.HttpServletRequest;

import org.hamcrest.Description;

import se.dannej.fakehttpserver.FakeHttpServletRequest;
import se.dannej.fakehttpserver.FakeHttpServletResponse;
import se.dannej.fakehttpserver.Header;
import se.dannej.fakehttpserver.JettyFakeServer;


/**
 * Invoked when none of the expectations for {@link JettyFakeServer} matched a request. 
 */
public class UnexpectedRequestExpectation extends Expectation {
	private StringBuilder message = new StringBuilder();
	private List<HttpServletRequest> unexpectedRequests = new ArrayList<HttpServletRequest>();

	@Override
	public boolean handle(FakeHttpServletRequest request, FakeHttpServletResponse response) {
		addMessage(request);
		this.unexpectedRequests.add(request);
		return true;
	}

	@Override
	public boolean isSatisfied(Description description) {
		if (!unexpectedRequests.isEmpty()) {
			description.appendText("Unexpected request(s):\n");
			description.appendText(message.toString());
			
			return false;
		}
		return true;
	}

	private void addMessage(FakeHttpServletRequest request) {
		if (message.length() > 0) {
			message.append("\n");
		}
		message.append(describeRequest(request));
	}
	
	private String describeRequest(FakeHttpServletRequest request) {
		return String.format("%s to %s (headers: %s)", request.getMethod(), request.getRequestURI(), commaSeparated(request.getHeaders()));
	}
	
	private String commaSeparated(Map<String,List<Header>> headers) {
		StringBuilder sb = new StringBuilder();
		
		for (Entry<String, List<Header>> entry : headers.entrySet()) {
			if (sb.length() > 0) {
				sb.append(", ");
			}
			
			sb.append("[" + entry.getKey() + ": ");
			List<Header> headerValues = entry.getValue();
			if (headerValues.size() > 0) {
				sb.append(headerValues.get(0).getValue());
				for (int i=1; i<headerValues.size(); i++) {
					sb.append(", ").append(headerValues.get(i).getValue());
				}
			}
			sb.append("]");
		}
		
		return sb.toString();
	}
}
