/*
 * Copyright 2011 Daniel Josefsson
 *
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package se.dannej.fakehttpserver.expect.matcher;

import java.util.ArrayList;
import java.util.List;
import java.util.Map.Entry;
import java.util.Set;

import org.hamcrest.Description;
import org.hamcrest.Matcher;

import se.dannej.fakehttpserver.FakeHttpServletRequest;
import se.dannej.fakehttpserver.Header;

/**
 * Class containing matchers for headers.
 */
public class HeaderMatchers {

	/**
	 * @param name header name
	 * @param value header value
	 * @return a header matcher, ignoring case on header name.
	 */
	public static RequestMatcher headerMatcher(final String name, final String value) {
		return new RequestMatcher() {
			@Override
			public boolean matchesSafely(FakeHttpServletRequest request) {
				for (Header header : request.getHeadersWithName(name)) {
					if (value.equals(header.getValue())) {
						return true;
					}
				}
				return false;
			}
			
			@Override
			public void describeTo(Description description) {
				description.appendText(String.format("header [%s,%s]", name, value));
			}
		};
	}

	/**
	 * @param nameMatcher header name matcher
	 * @param valueMatcher header value matcher
	 * @return a header matcher
	 */
	public static RequestMatcher headerMatcher(final Matcher<String> nameMatcher, final Matcher<String> valueMatcher) {
		return new RequestMatcher() {
			@Override
			public boolean matchesSafely(FakeHttpServletRequest request) {
				List<Header> headersWithMatchingNames = headersWithNameMatching(nameMatcher, request);
				
				for (Header header : headersWithMatchingNames) {
					if (valueMatcher.matches(header.getValue())) {
						return true;
					}
				}
				return false;
			}
			
			@Override
			public void describeTo(Description description) {
				description.appendText("header [");
				nameMatcher.describeTo(description);
				description.appendText(", ");
				valueMatcher.describeTo(description);
				description.appendText("]");
			}
		};
	}
	
	private static List<Header> headersWithNameMatching(Matcher<String> nameMatcher, FakeHttpServletRequest request) {
		List<Header> matchingHeaders = new ArrayList<Header>();
		
		Set<Entry<String, List<Header>>> entrySet = request.getHeaders().entrySet();
		
		for (Entry<String, List<Header>> entry : entrySet) {
			if (nameMatcher.matches(entry.getKey())) {
				matchingHeaders.addAll(entry.getValue());
			}
		}
		
		return matchingHeaders;
	}
}
