/*
 * Copyright 2011 Daniel Josefsson
 *
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package se.dannej.fakehttpserver.expect.matcher;

import static org.hamcrest.core.IsEqual.equalTo;
import static se.dannej.fakehttpserver.expect.matcher.HeaderMatchers.headerMatcher;

import java.io.IOException;

import org.hamcrest.Description;
import org.hamcrest.Matcher;

import se.dannej.fakehttpserver.FakeHttpServletRequest;

/**
 * Convenience class for creating {@link RequestMatcher}s.
 * @see Requests.
 */
public class RequestMatchers {
	
	/**
	 * Matches request path against string.
	 * @param path
	 * @return
	 */
	public static RequestMatcher path(String path) {
		return pathMatcher(equalTo(path));
	}

	/**
	 * Matches request path using {@link Matcher}.
	 * @param pathMatcher
	 * @return
	 */
	public static RequestMatcher path(Matcher<String> pathMatcher) {
		return pathMatcher(pathMatcher);
	}
	
	/**
	 * Matches request header against strings.
	 * @param name
	 * @param value
	 * @return
	 */
	public static RequestMatcher header(String name, String value) {
		return headerMatcher(name, value);
	}
	
	/**
	 * Matches request header using {@link Matcher}.
	 * @param nameMatcher
	 * @param valueMatcher
	 * @return
	 */
	public static RequestMatcher header(Matcher<String> nameMatcher, Matcher<String> valueMatcher) {
		return headerMatcher(nameMatcher, valueMatcher);
	}
	
	/**
	 * Matches content type against string.
	 * @param contentType
	 * @return
	 */
	public static RequestMatcher contentType(String contentType) {
		return contentTypeMatcher(equalTo(contentType));
	}
	
	/**
	 * Matches content-type using {@link Matcher}.
	 * @param contentTypeMatcher
	 * @return
	 */
	public static RequestMatcher contentType(Matcher<String> contentTypeMatcher) {
		return contentTypeMatcher(contentTypeMatcher);
	}

	/**
	 * Matches content against string.
	 * @param content
	 * @return
	 */
	public static RequestMatcher content(String content) {
		return content(equalTo(content.getBytes()));
	}

	/**
	 * Matches content against string.
	 * @param content
	 * @return
	 */
	public static RequestMatcher content(byte[] content) {
		return content(equalTo(content));
	}

	/**
	 * Matches content using hamcrest Matcher.
	 * @param contentMatcher
	 * @return
	 */
	public static RequestMatcher content(Matcher<byte[]> contentMatcher) {
		return contentMatcher(contentMatcher);
	}
	
	/**
	 * Matches content as a string using hamcrest Matcher.
	 * @param contentMatcher
	 * @return
	 */
	public static RequestMatcher stringContent(Matcher<String> contentMatcher) {
		return stringContentMatcher(contentMatcher);
	}
	
	private static RequestMatcher pathMatcher(final Matcher<String> pathMatcher) {
		return new RequestMatcher() {
			@Override
			public boolean matchesSafely(FakeHttpServletRequest request) {
				return pathMatcher.matches(request.getRequestURI());
			}

			@Override
			public void describeTo(Description description) {
				description.appendText("path is ");
				pathMatcher.describeTo(description);
			}
		};
	}

	private static RequestMatcher contentTypeMatcher(final Matcher<String> contentTypeMatcher) {
		return new RequestMatcher() {
			@Override
			public boolean matchesSafely(FakeHttpServletRequest request) {
				return contentTypeMatcher.matches(request.getContentType());
			}
			
			@Override
			public void describeTo(Description description) {
				description.appendText("content-type is ");
				contentTypeMatcher.describeTo(description);
			}
		};
	}

	private static RequestMatcher contentMatcher(final Matcher<byte[]> contentMatcher) {
		return new RequestMatcher() {
			@Override
			public boolean matchesSafely(FakeHttpServletRequest request) {
				try {
					return contentMatcher.matches(request.getContent());
				} catch (IOException e) {
					e.printStackTrace();
					return false;
				}
			}
			
			@Override
			public void describeTo(Description description) {
				description.appendText("content is ");
				contentMatcher.describeTo(description);
			}
		};
	}

	private static RequestMatcher stringContentMatcher(final Matcher<String> contentMatcher) {
		return new RequestMatcher() {
			@Override
			public boolean matchesSafely(FakeHttpServletRequest request) {
				try {
					String actualContent = new String(request.getContent());
					return contentMatcher.matches(actualContent);
				} catch (IOException e) {
					e.printStackTrace();
					return false;
				}
			}
			
			@Override
			public void describeTo(Description description) {
				description.appendText("content is ");
				contentMatcher.describeTo(description);
			}
		};
	}
}
